/*
 * Proiect.c
 *
 * Created: 5/17/2018 2:43:50 PM
 * Author : Bogdan
 */ 

#define F_CPU 16000000
#include <avr/io.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#define USART_H_
#define LOOP_CYCLES 8 //Number of cycles that the loop takes
#define us(num) (num/(LOOP_CYCLES*(1/(F_CPU/1000000.0))))

#include <util/delay.h>
#include<avr/interrupt.h>

/* Thermometer Connections (At your choice) */
#define THERM_PORT PORTA
#define THERM_DDR DDRA
#define THERM_PIN PINA
#define THERM_DQ PA7

/* Utils */
#define THERM_INPUT_MODE() THERM_DDR&=~(1<<THERM_DQ)
#define THERM_OUTPUT_MODE() THERM_DDR|=(1<<THERM_DQ)
#define THERM_LOW() THERM_PORT&=~(1<<THERM_DQ)
#define THERM_HIGH() THERM_PORT|=(1<<THERM_DQ)

#define THERM_CMD_CONVERTTEMP 0x44
#define THERM_CMD_RSCRATCHPAD 0xbe
#define THERM_CMD_WSCRATCHPAD 0x4e
#define THERM_CMD_CPYSCRATCHPAD 0x48
#define THERM_CMD_RECEEPROM 0xb8
#define THERM_CMD_RPWRSUPPLY 0xb4
#define THERM_CMD_SEARCHROM 0xf0
#define THERM_CMD_READROM 0x33
#define THERM_CMD_MATCHROM 0x55
#define THERM_CMD_SKIPROM 0xcc
#define THERM_CMD_ALARMSEARCH 0xec

inline __attribute__((gnu_inline)) void therm_delay(uint16_t delay){
	while(delay--) asm volatile("nop");
}

uint8_t therm_reset(){
	uint8_t i;

	//Pull line low and wait for 480uS
	THERM_LOW();
	THERM_OUTPUT_MODE();
	therm_delay(us(480));
	
	//Release line and wait for 60uS
	THERM_INPUT_MODE();
	therm_delay(us(60));
	
	//Store line value and wait until the completion of 480uS period
	i=(THERM_PIN & (1<<THERM_DQ));
	therm_delay(us(420));
	
	//Return the value read from the presence pulse (0=OK, 1=WRONG)
	return i;
}

void therm_write_bit(uint8_t bit){
	//Pull line low for 1uS
	THERM_LOW();
	THERM_OUTPUT_MODE();
	therm_delay(us(1));
	
	//If we want to write 1, release the line (if not will keep low)
	if(bit) THERM_INPUT_MODE();
	
	//Wait for 60uS and release the line
	therm_delay(us(60));
	THERM_INPUT_MODE();
}

uint8_t therm_read_bit(void){
	uint8_t bit=0;
	
	//Pull line low for 1uS
	THERM_LOW();
	THERM_OUTPUT_MODE();
	therm_delay(us(1));
	
	//Release line and wait for 14uS
	THERM_INPUT_MODE();
	therm_delay(us(14));
	
	//Read line value
	if(THERM_PIN&(1<<THERM_DQ)) bit=1;
	
	//Wait for 45uS to end and return read value
	therm_delay(us(45));
	return bit;
}

uint8_t therm_read_byte(void){
	uint8_t i=8, n=0;
	while(i--){
		n>>=1;
		n|=(therm_read_bit()<<7);
	}

	return n;
}
void therm_write_byte(uint8_t byte){
	uint8_t i=8;
	while(i--){
		//Write actual bit and shift one position right to make
		//the next bit ready
		therm_write_bit(byte&1);
		byte>>=1;
	}
}


#define THERM_DECIMAL_STEPS_12BIT 625 //.0625
void therm_read_temperature(char *buffer){
	// Buffer length must be at least 12bytes long! ["+XXX.XXXX C"]
	uint8_t temperature[2];
	int8_t digit;
	uint16_t decimal;
	
	//Reset, skip ROM and start temperature conversion
	therm_reset();
	therm_write_byte(THERM_CMD_SKIPROM);
	therm_write_byte(THERM_CMD_CONVERTTEMP);
	
	//Wait until conversion is complete
	while(!therm_read_bit());
	
	//Reset, skip ROM and send command to read Scratchpad
	therm_reset();
	therm_write_byte(THERM_CMD_SKIPROM);
	therm_write_byte(THERM_CMD_RSCRATCHPAD);
	
	//Read Scratchpad (only 2 first bytes)
	temperature[0]=therm_read_byte();
	temperature[1]=therm_read_byte();
	therm_reset();
	
	//Store temperature integer digits and decimal digits
	digit=temperature[0]>>4;
	digit|=(temperature[1]&0x7)<<4;
	
	//Store decimal digits
	decimal=temperature[0]&0xf;
	decimal*=THERM_DECIMAL_STEPS_12BIT;
	
	//Format temperature into a string [+XXX.XXXX C]
	sprintf(buffer, "%+d.%04u C", digit, decimal);
}



/*
 * Functie de initializare a controllerului USART
 */
void USART0_init()
{
    /* seteaza baud rate la 9600 */
    UBRR0H = 0;
    UBRR0L = 103;

    /* porneste transmitatorul */
    UCSR0B = (1<<TXEN0) | (1<<RXEN0);

    /* seteaza formatul frame-ului: 8 biti de date, 1 biti de stop, paritate para */
    UCSR0C &= ~(1<<USBS0);
    UCSR0C |= (2<<UCSZ00);
    UCSR0C |= (1<<UPM01);
}

/*
 * Functie ce transmite un caracter prin USART
 *
 * @param data - caracterul de transmis
 */
void USART0_transmit(char data)
{
    /* asteapta pana bufferul e gol */
    while(!(UCSR0A & (1<<UDRE0)));

    /* pune datele in buffer; transmisia va porni automat in urma scrierii */
    UDR0 = data;
}

/*
 * Functie ce primeste un caracter prin USART
 *
 * @return - caracterul primit
 */
char USART0_receive()
{
    /* asteapta cat timp bufferul e gol */
    while(!(UCSR0A & (1<<RXC0)));

    /* returneaza datele din buffer */
    return UDR0;
}

/*
 * Functie ce transmite un sir de caractere prin USART
 *
 * @param data - sirul (terminat cu '\0') de transmis
 */
void USART0_print(const char *data)
{
    while(*data != '\0')
        USART0_transmit(*data++);
}


int command = -1;
char buffer[20];
int buffer_index = 0;
int read_done = 0;
int select_temp = 0;
int enter_temp = 0;
int select_turation = 0;
float starting_temp = 10000;
int int_procent = 0;

//Each time a command is received, an interrupt is issued
ISR(USART0_RX_vect)
{
	//Read the full string
	char received = UDR0;
	if (received != '\n') {
		buffer[buffer_index++] = received;
	}
	else {
		buffer[--buffer_index] = '\0';
		if(strcmp(buffer, "start") == 0)
			command = 0;
		else if (strcmp(buffer, "temp") == 0)
			command = 1;
		else if (strcmp(buffer, "stop") == 0)
			command = 3;
		else if (strcmp(buffer, "+") == 0)
			command = 5;
		else if (strcmp(buffer, "-") == 0)
			command = 6;
		else if (strcmp(buffer, "set temp") == 0)
			select_temp = 1;
		else if(enter_temp) {
			enter_temp = 0;
			command = 4;
		}

		if(!select_temp)
			read_done = 1;
	}
}

//Initialize timer1 as fast pwm no prescaling
void timer1_init()
{
	TCCR1A = (1 << WGM10) | (1 << COM1A1);
	TCCR1B = (1 << CS10) | (1 << WGM12);
	DDRD = 0xFF;
	PORTD &= ~(1 << PD5);
	OCR1A=0;
}


//The program is a big state machine
int main(void) {
DDRD |= (1 << PD7);

//initialize the usart
USART0_init();

char temperature[20];

//Set timer for pwm
timer1_init();

//Initial message
USART0_print("Please enter command:\n");

while(1) {
//enable interrupts and interrupts when something is received on rx
sei();
UCSR0B |= (1 << RXCIE0);

//check if starting temp is lower, then turn on the fan
therm_read_temperature(temperature);
char current_temp[8];
int i;
for (i = 1; i < 8; i++) {
	current_temp[i - 1] = temperature[i];
}

current_temp[i] = '\0';

//used to check if input is float
if (atof(current_temp) > starting_temp){
	OCR1A = 255;
}

//used to check if input is int
if(atof(current_temp) == 0.0) {
	if(atoi(current_temp) > starting_temp){
		OCR1A = 255;
	}
}

//Select temp state 
if (select_temp) {
	select_temp = 0;
	USART0_print("Enter value for temp:\n");
	buffer_index = 0;
	enter_temp = 1;
}

//After the command was read, we analyze it
if (read_done){
	switch (command) {
		case 0: USART0_print("Starting fan\n"); 
				OCR1A = 255;
				int_procent = 255;
				command = -1; 
				break;
		
		case 1: USART0_print("The temperature is "); 
				therm_read_temperature(temperature);
				USART0_print(temperature);
				USART0_print("\n");
				command = -1; 
				break;

		case 3: USART0_print("Stopping fan\n");
				OCR1A = 0;
				int_procent = 0;
				starting_temp = 10000;
				command = -1;
				break;

		case 4: USART0_print("Temperature for turn on set to ");
				starting_temp = atof(buffer);
				USART0_print(buffer);
				USART0_print("\n");
				command = -1;
				break;

		case 5: USART0_print("Power has gone up\n");
				if (int_procent == 0){
					OCR1A = 105;
					int_procent = 105;
				}
				else {
					if(int_procent+50 < 255){
						int_procent += 50;
						OCR1A = int_procent;
					}
					else{
						int_procent = 255;
						OCR1A = 255;
						USART0_print("Maximum power\n");
					}
				}
				command = -1;
				break;

		case 6: USART0_print("Power has gone down\n");
				if (int_procent - 50 < 100){
					OCR1A = 0;
					int_procent = 0;
					USART0_print("Stopping fan\n");
				} 
					else {
						int_procent -= 50;
						OCR1A = int_procent;
					}
				command = -1;
				break;

		default: USART0_print("Enter one of the following commands:\n \"start\"\n \"stop\"\n \"temp\"\n \"set temp\" \n \"power up (+)\"\n\"power down (-)\"\n"); 
				command = -1; 
				break;
	}
	
	read_done = 0;
	buffer_index = 0;
}
}
return 0;
}

